@@ -67,9 +67,9 @@
 package PDL::PP::Rule;
 
 use strict;
+use warnings;
 
 use Carp;
-our @CARP_NOT;
 
 use overload ("\"\"" => \&PDL::PP::Rule::stringify);
 sub stringify {
@@ -219,8 +219,6 @@ package PDL::PP::Rule::Croak;
 # incompatible arguments.
 our @ISA = qw(PDL::PP::Rule);
 use Carp;
-our @CARP_NOT;
-
 
 sub new {
     croak('Usage: PDL::PP::Rule::Croak->new(["incompatible", "arguments"], "Croaking message")')
@@ -234,11 +232,8 @@ sub apply {
 }
 
 package PDL::PP::Rule::Returns;
-
 use strict;
-
 use Carp;
-our @CARP_NOT;
 
 our @ISA = qw (PDL::PP::Rule);
 
@@ -318,10 +313,7 @@ sub new {
 package PDL::PP::Rule::InsertName;
 
 use strict;
-
 use Carp;
-our @CARP_NOT;
-
 our @ISA = qw (PDL::PP::Rule);
 
 # This class does not treat return values of "DO NOT SET!!"
@@ -384,10 +376,7 @@ sub apply {
 package PDL::PP::Rule::Substitute;
 
 use strict;
-
 use Carp;
-our @CARP_NOT;
-
 our @ISA = qw (PDL::PP::Rule);
 
 sub badflag_isset {
@@ -471,9 +460,7 @@ sub new {
 package PDL::PP::Rule::Substitute::Usual;
 
 use strict;
-
 use Carp;
-our @CARP_NOT;
 
 our @ISA = qw (PDL::PP::Rule::Substitute);
 
@@ -513,9 +500,7 @@ sub extract_args {
 package PDL::PP::Rule::MakeComp;
 
 use strict;
-
 use Carp;
-our @CARP_NOT;
 
 our @ISA = qw (PDL::PP::Rule);
 
@@ -614,7 +599,7 @@ our $macros_xs = <<'EOF';
 #define PDL_XS_PERLINIT(name, to_push, method) \
   if (strcmp(objname,"PDL") == 0) { PDL_COMMENT("shortcut if just PDL") \
      name ## _SV = sv_newmortal(); \
-     name = PDL->null(); \
+     name = PDL->pdlnew(); \
      if (!name) PDL->pdl_barf("Error making null pdl"); \
      PDL->SetSV_PDL(name ## _SV, name); \
      if (bless_stash) name ## _SV = sv_bless(name ## _SV, bless_stash); \
@@ -637,6 +622,16 @@ our $macros_xs = <<'EOF';
     } else { \
       XSRETURN(0); \
     }
+
+#define PDL_XS_INPLACE(in, out) \
+    if (in->state & PDL_INPLACE && (out != in)) { \
+	in->state &= ~PDL_INPLACE; PDL_COMMENT("unset") \
+	out = in; \
+	PDL->SetSV_PDL(out ## _SV,out); \
+    }
+
+#define PDL_XS_INPLACE_CHECK(in) \
+    if (in->state & PDL_INPLACE) barf("inplace input but output given");
 EOF
 
 our $header_c = pp_line_numbers(__LINE__, <<'EOF');
@@ -659,16 +654,6 @@ PDL_COMMENT("just think of it as a C multiline comment like:                ")
 PDL_COMMENT("                                                               ")
 PDL_COMMENT("   /* Memory access */                                         ")
 
-#define PDL_XS_INPLACE(in, out) \
-    if (in->state & PDL_INPLACE && (out != in)) { \
-	in->state &= ~PDL_INPLACE; PDL_COMMENT("unset") \
-	out = in; \
-	PDL->SetSV_PDL(out ## _SV,out); \
-    }
-
-#define PDL_XS_INPLACE_CHECK(in) \
-    if (in->state & PDL_INPLACE) barf("inplace input but output given");
-
 #define PDL_FREE_CODE(trans, destroy, comp_free_code, ntpriv_free_code) \
     if (destroy) { \
 	comp_free_code \
@@ -743,7 +728,6 @@ $::PP_VERBOSE    = 0;
 our $done = 0;  # pp_done has not been called yet
 
 use Carp;
-our @CARP_NOT;
 
 sub nopm { $::PDLPACK eq 'NONE' } # flag that we don't want to generate a PM
 
@@ -776,8 +760,11 @@ sub import {
 sub list_functions {
   my ($file) = @_;
   my @funcs;
+  undef &PDL::PP::pp_def;
   local *PDL::PP::pp_def = sub { push @funcs, (_pp_parsename($_[0]))[0]};
+  undef &PDL::PP::pp_done;
   local *PDL::PP::pp_done = sub {};
+  $_ = '' for $::PDLMOD, $::CALLPACK, $::PDLOBJ; # stop warnings
   require File::Spec::Functions;
   do ''.File::Spec::Functions::rel2abs($file);
   die $@ if $@;
@@ -822,18 +809,19 @@ sub pp_addhdr {
 }
 
 sub pp_addpm {
- 	my $pm = shift;
- 	my $pos;
- 	if (ref $pm) {
- 	  my $opt = $pm;
- 	  $pm = shift;
- 	  croak "unknown option" unless defined $opt->{At} &&
- 	    $opt->{At} =~ /^(Top|Bot|Middle)$/;
- 	  $pos = $opt->{At};
- 	} else {
- 	  $pos = 'Middle';
- 	}
- 	$::PDLPM{$pos} .= "$pm\n\n";
+	my $pm = shift;
+	my $pos;
+	if (ref $pm) {
+	  my $opt = $pm;
+	  $pm = shift;
+	  croak "unknown option" unless defined $opt->{At} &&
+	    $opt->{At} =~ /^(Top|Bot|Middle)$/;
+	  $pos = $opt->{At};
+	} else {
+	  $pos = 'Middle';
+	}
+	my @c = caller;
+	$::PDLPM{$pos} .= _pp_line_number_file(@c[1,2], $pm) . "\n\n";
 }
 
 sub pp_add_exported {
@@ -892,14 +880,15 @@ sub pp_addxs {
 #   FirstKey => ...,
 #   Code => pp_line_numbers(__LINE__, $x . $y . $c),
 #   OtherKey => ...
-sub pp_line_numbers ($$) {
-	my ($line, $string) = @_;
+sub pp_line_numbers {
+  _pp_line_number_file((caller)[1], @_);
+}
+sub _pp_line_number_file {
+	my ($filename, $line, $string) = @_;
+	confess "pp_line_numbers called with undef" if !defined $string;
 	# The line needs to be incremented by one for the bookkeeping to work
 	$line++;
-	# Get the source filename using caller()
-	my (undef, $filename) = caller;
-	# Escape backslashes:
-	$filename =~ s/\\/\\\\/g;
+	$filename =~ s/\\/\\\\/g; # Escape backslashes
 	my @to_return = "\nPDL_LINENO_START $line \"$filename\"\n";
 	# Look for threadloops and loops and add # line directives
 	foreach (split (/\n/, $string)) {
@@ -954,7 +943,7 @@ sub printxsc {
   my $text = join '',@_;
   if (defined $file) {
     (my $mod_underscores = $::PDLMOD) =~ s#::#_#g;
-    $text = join '', sprintf($PDL::PP::header_c, $mod_underscores, $PP::boundscheck), $::PDLXSC_header, $text;
+    $text = join '', sprintf($PDL::PP::header_c, $mod_underscores, $PP::boundscheck), $::PDLXSC_header//'', $text;
     _write_file($file, $text);
   } else {
     $::PDLXSC .= $text;
@@ -971,16 +960,17 @@ sub pp_done {
         (my $mod_underscores = $::PDLMOD) =~ s#::#_#g;
         my $text = join '',
           sprintf($PDL::PP::header_c, $mod_underscores, $PP::boundscheck),
-          $::PDLXSC,
+          $::PDLXSC//'',
           $PDL::PP::macros_xs, sprintf($PDL::PP::header_xs,
             $::PDLMOD, $::PDLOBJ, $::PDLXS,
-            $pdl_boot, $::PDLXSBOOT, $PP::boundscheck,
+            $pdl_boot, $::PDLXSBOOT//'', $PP::boundscheck,
           );
         _write_file("$::PDLPREF.xs", $text);
         return if nopm;
 	$::PDLPMISA = "'".join("','",@::PDLPMISA)."'";
 	$::PDLBEGIN = "BEGIN {\n$::PDLBEGIN\n}"
 		unless $::PDLBEGIN =~ /^\s*$/;
+        $::PDLMODVERSION //= '';
         $::FUNCSPOD = $::DOCUMENTED ? "\n\n=head1 FUNCTIONS\n\n=cut\n\n" : '';
         _write_file("$::PDLPREF.pm", join "\n\n", <<EOF, $::PDLBEGIN, $::PDLPM{Top}, $::FUNCSPOD, @::PDLPM{qw(Middle Bot)}, '# Exit with OK status', "1;\n");
 #
@@ -1047,7 +1037,7 @@ sub pp_def {
 	croak("ERROR: No FreeFunc for pp_def=$name!\n")
 	  unless exists $obj{FreeFunc};
 
-	my $ctext = join("\n\n",@obj{'StructDecl','RedoDimsFunc',
+	my $ctext = join("\n\n",grep $_, @obj{'StructDecl','RedoDimsFunc',
 		'ReadDataFunc','WriteBackDataFunc',
 		'FreeFunc',
 		'VTableDef','RunFunc',
@@ -1063,10 +1053,10 @@ EOF
 	  PDL::PP->printxsc(undef, $ctext);
 	}
 	PDL::PP->printxs($obj{NewXSCode});
-	pp_add_boot($obj{BootSetNewXS});
+	pp_add_boot($obj{BootSetNewXS}) if $obj{BootSetNewXS};
 	PDL::PP->pp_add_exported($name);
 	PDL::PP::pp_addpm("\n".$obj{PdlDoc}."\n") if $obj{PdlDoc};
-	PDL::PP::pp_addpm($obj{PMCode});
+	PDL::PP::pp_addpm($obj{PMCode}) if defined $obj{PMCode};
 	PDL::PP::pp_addpm($obj{PMFunc}."\n");
 
 	print "*** Leaving pp_def for $name\n" if $::PP_VERBOSE;
@@ -1422,7 +1412,7 @@ $PDL::PP::deftbl =
         # definition of the EQUIVCPOFFS macro - see the Code rule
         PDL::PP::pp_line_numbers(__LINE__,
             'PDL_Indx i;
-             for(i=0; i<$CHILD_P(nvals); i++)  {
+             for(i=0; i<$PDL(CHILD)->nvals; i++)  {
                 $EQUIVCPOFFS(i,i);
              }'),
         1, 1, 1);
@@ -1639,11 +1629,11 @@ EOD
        'Sets GenericTypes flag to all real types known to PDL::Types',
        sub {[PDL::Types::ppdefs()]}),
 
-   PDL::PP::Rule->new("ExtraGenericLoops", "FTypes",
-       'Makes ExtraGenericLoops identical to FTypes if the latter exists and the former does not',
+   PDL::PP::Rule->new("ExtraGenericSwitches", "FTypes",
+       'Makes ExtraGenericSwitches identical to FTypes if the latter exists and the former does not',
        sub {return $_[0]}),
-   PDL::PP::Rule::Returns->new("ExtraGenericLoops", [],
-		'Sets ExtraGenericLoops to an empty hash if it does not already exist', {}),
+   PDL::PP::Rule::Returns->new("ExtraGenericSwitches", [],
+       'Sets ExtraGenericSwitches to an empty hash if it does not already exist', {}),
 
    PDL::PP::Rule::InsertName->new("VTableName", 'pdl_${name}_vtable'),
 
@@ -1818,31 +1808,39 @@ EOD
 
    PDL::PP::Rule->new("VarArgsXSHdr",
       ["Name","SignatureObj",
-       "HdrCode","InplaceCode","InplaceCheck",\"CallCopy",\"Bitwise"],
+       "HdrCode","InplaceCode","InplaceCheck",\"CallCopy",\"OtherParsDefaults"],
       'XS code to process input arguments based on supplied Pars argument to pp_def; not done if GlobalNew or PMCode supplied',
       sub {
         my($name,$sig,
-           $hdrcode,$inplacecode,$inplacecheck,$callcopy,$bitwise) = @_;
-        # Don't do var args processing if the user has pre-defined pmcode
+           $hdrcode,$inplacecode,$inplacecheck,$callcopy,$defaults) = @_;
+        my $optypes = $sig->otherobjs(1);
+        my @args = $sig->alldecls(0, 1);
+        my %other  = map +($_ => exists($$optypes{$_})), @args;
+        if (keys %{ $defaults ||= {} } < keys %other) {
+          my $default_seen = '';
+          for (@args) {
+            $default_seen = $_ if exists $defaults->{$_};
+            confess "got default-less arg '$_' after default-ful arg '$default_seen'"
+              if $default_seen and !exists $defaults->{$_};
+          }
+        }
         my $ci = '  ';  # current indenting
         my $pars = join "\n",map "$ci$_ = 0;", $sig->alldecls(1, 0);
-        my @args = $sig->alldecls(0, 1);
         my %out = map +($_=>1), $sig->names_out_nca;
         my %outca = map +($_=>1), $sig->names_oca;
         my %tmp = map +($_=>1), $sig->names_tmp;
-        my $optypes = $sig->otherobjs(1);
-        my %other  = map { $_ => exists($$optypes{$_}) } @args;
-        # remember, othervars *are* input vars
-        my $nout   = (grep { $_ } values %out);
-        my $noutca = (grep { $_ } values %outca);
-        my $nother = (grep { $_ } values %other);
-        my $ntmp   = (grep { $_ } values %tmp);
+        # remember, otherpars *are* input vars
+        my $nout   = grep $_, values %out;
+        my $noutca = grep $_, values %outca;
+        my $nother = grep $_, values %other;
+        my $ntmp   = grep $_, values %tmp;
         my $ntot   = @args;
         my $nmaxonstack = $ntot - $noutca;
         my $nin    = $ntot - ($nout + $noutca + $ntmp);
         my $ninout = $nin + $nout;
         my $nallout = $nout + $noutca;
-        my $usageargs = join (",", @args);
+        my $ndefault = keys %$defaults;
+        my $usageargs = join ",", map exists $defaults->{$_} ? "$_=$defaults->{$_}" : $_, @args;
         # Generate declarations for SV * variables corresponding to pdl * output variables.
         # These are used in creating output and temp variables.  One variable (ex: SV * outvar1_SV;)
         # is needed for each output and output create always argument
@@ -1903,10 +1901,14 @@ EOD
         }
         # clause for reading in input and creating output and temp vars
         my $clause3 = '';
+        my $defaults_rawcond = $ndefault ? "items == ($nin-$ndefault)" : '';
         $cnt = 0;
         foreach my $x (@args) {
             if ($other{$x}) {
-                $clause3 .= "$ci$x = " . typemap($x, $$optypes{$x}, "ST($cnt)") . ";\n";
+                my $setter = typemap($x, $$optypes{$x}, "ST($cnt)");
+                $clause3 .= "$ci$x = " . (exists $defaults->{$x}
+                  ? "($defaults_rawcond) ? ($defaults->{$x}) : ($setter)"
+                  : $setter) . ";\n";
                 $cnt++;
             } elsif ($out{$x} || $tmp{$x} || $outca{$x}) {
                 push (@create, $x);
@@ -1917,14 +1919,13 @@ EOD
         }
         # Add code for creating output variables via call to 'initialize' perl routine
         $clause3 .= callPerlInit (\@create, $ci, $callcopy); @create = ();
-        # Bitwise ops may get five args
-        my $bitwise_cond = $bitwise ? " || items == 5" : '';
+        my $defaults_cond = $ndefault ? " || $defaults_rawcond" : '';
         $clause3 = <<EOF . $clause3;
-  else if (items == $nin$bitwise_cond) { PDL_COMMENT("only input variables on stack, create outputs and temps")
+  else if (items == $nin$defaults_cond) { PDL_COMMENT("only input variables on stack, create outputs and temps")
     nreturn = $nallout;
 EOF
-        $clause3 = '' if $nmaxonstack == $nin and !$bitwise;
-        my $clause3_coda = $clause3 ? '}' : '';
+        $clause3 = '' if $nmaxonstack == $nin;
+        my $clause3_coda = $clause3 ? '  }' : '';
         PDL::PP::pp_line_numbers(__LINE__, <<END);
 
 void
@@ -1934,7 +1935,7 @@ $name(...)
 $svdecls
 $pars
  PPCODE:
-  if (items != $nmaxonstack && !(items == $nin$bitwise_cond) && items != $ninout)
+  if (items != $nmaxonstack && !(items == $nin$defaults_cond) && items != $ninout)
     croak (\"Usage:  PDL::$name($usageargs) (you may leave temporaries or output variables out of list)\");
   PDL_XS_PACKAGEGET
 $clause_inputs
@@ -2191,7 +2192,7 @@ EOF
    PDL::PP::Rule->new(["NewXSCode","BootSetNewXS","NewXSInPrelude"],
       ["NewXSHdr", "NewXSCHdrs", "RunFuncCall"],
       "Non-varargs XS code when GlobalNew given",
-      sub {make_xs_code('CODE:',' XSRETURN(0);',@_)}),
+      sub {(undef,(make_xs_code('CODE:',' XSRETURN(0);',@_))[1..2])}),
    # if PMCode supplied, no var-args stuff
    PDL::PP::Rule->new(["NewXSCode","BootSetNewXS","NewXSInPrelude"],
       ["PMCode","NewXSHdr", \"NewXSCHdrs", "RunFuncCall"],
