@@ -1,6 +1,8 @@
 # This is the new Filter::Simple engine for PDL::NiceSlice
 #
 use Filter::Simple;
+use strict;
+use warnings;
 
 FILTER_ONLY
    all => sub { s/\r\n/\n/g if $^V lt v5.14.0 and $^O eq 'MSWin32'; },
@@ -11,6 +13,7 @@ FILTER_ONLY
       $text2 = perldlpp('PDL::NiceSlice', $text1);
       ## print STDERR "**************** Output: $text2\n";
       $_ = $text2;
-   };
+   },
+   all => sub { print if $PDL::NiceSlice::debug_filter };
 
 1;
